@@ -3,7 +3,7 @@ use strict;
 use warnings;
 package YAML::PP::Parser;
 
-our $VERSION = '0.027'; # VERSION
+our $VERSION = '0.028'; # VERSION
 
 use constant TRACE => $ENV{YAML_PP_TRACE} ? 1 : 0;
 use constant DEBUG => ($ENV{YAML_PP_DEBUG} || $ENV{YAML_PP_TRACE}) ? 1 : 0;
@@ -1266,6 +1266,21 @@ sub cb_flowkey_quoted {
     $self->scalar_event($info);
 }
 
+sub cb_empty_flowmap_key_value {
+    my ($self, $token) = @_;
+    $self->cb_empty_flow_mapkey($token);
+    $self->cb_empty_flowmap_value;
+    $self->cb_flow_comma;
+}
+
+sub cb_end_empty_flowmap_key_value {
+    my ($self, $token) = @_;
+    $self->cb_empty_flow_mapkey($token);
+    $self->cb_empty_flowmap_value;
+    $self->cb_flow_comma;
+    $self->cb_end_flowmap;
+}
+
 sub cb_empty_flowmap_value {
     my ($self, $token) = @_;
     my $stack = $self->event_stack;
@@ -1280,6 +1295,18 @@ sub cb_empty_flowmap_value {
     $self->scalar_event($info);
 }
 
+sub cb_empty_flowseq_comma {
+    my ($self, $token) = @_;
+    $self->cb_empty_flowmap_value($token);
+    $self->cb_flow_comma;
+}
+
+sub cb_empty_flowseq_end {
+    my ($self, $token) = @_;
+    $self->cb_empty_flowmap_value($token);
+    $self->cb_end_flowseq;
+}
+
 sub cb_insert_map_alias {
     my ($self, $res) = @_;
     my $stack = $self->event_stack;
